Module: Sisimai::Lhost::Activehunter

Defined in:
lib/sisimai/lhost/activehunter.rb

Overview

Sisimai::Lhost::Activehunter decodes a bounce email which created by QUALITIA Active!hunter www.qualitia.com/jp/product/ah/function.html. Methods in the module are called from only Sisimai::Message.

Constant Summary collapse

Indicators =
Sisimai::Lhost.INDICATORS
Boundaries =
['Content-Type: message/rfc822'].freeze
StartingOf =
{message: ['  ----- The following addresses had permanent fatal errors -----']}.freeze

Class Method Summary collapse

Class Method Details

.descriptionObject



68
# File 'lib/sisimai/lhost/activehunter.rb', line 68

def description; return 'TransWARE Active!hunter'; end

.inquire(mhead, mbody) ⇒ Hash, Nil

This method is abstract.

decodes the bounce message from QUALITIA Active!hunter

Parameters:

  • mhead (Hash)

    Message headers of a bounce email

  • mbody (String)

    Message body of a bounce email

Returns:

  • (Hash)

    Bounce data list and message/rfc822 part

  • (Nil)

    it failed to decode or the arguments are missing



18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
# File 'lib/sisimai/lhost/activehunter.rb', line 18

def inquire(mhead, mbody)
  # :from    => %r/\A"MAILER-DAEMON"/,
  # :subject => %r/FAILURE NOTICE :/,
  return nil if mhead['x-ahmailid'].nil?

  dscontents = [Sisimai::Lhost.DELIVERYSTATUS]; v = nil
  emailparts = Sisimai::RFC5322.part(mbody, Boundaries)
  bodyslices = emailparts[0].split("\n")
  readcursor = 0      # (Integer) Points the current cursor position
  recipients = 0      # (Integer) The number of 'Final-Recipient' header

  while e = bodyslices.shift do
    # Read error messages and delivery status lines from the head of the email to the previous
    # line of the beginning of the original message.
    if readcursor == 0
      # Beginning of the bounce message or delivery status part
      readcursor |= Indicators[:deliverystatus] if e == StartingOf[:message][0]
      next
    end
    next if (readcursor & Indicators[:deliverystatus]) == 0 || e.empty?

    #  ----- The following addresses had permanent fatal errors -----
    #
    # >>> kijitora@example.org <kijitora@example.org>
    #
    #  ----- Transcript of session follows -----
    # 550 sorry, no mailbox here by that name (#5.1.1 - chkusr)
    v = dscontents[-1]

    if e.start_with?('>>> ') && e.index('@') > 1
      # >>> kijitora@example.org <kijitora@example.org>
      if v["recipient"] != ""
        # There are multiple recipient addresses in the message body.
        dscontents << Sisimai::Lhost.DELIVERYSTATUS
        v = dscontents[-1]
      end
      v['recipient'] = Sisimai::Address.s3s4(e[e.index('<'), e.size])
      v['diagnosis'] = ''
      recipients += 1
    else
      #  ----- Transcript of session follows -----
      # 550 sorry, no mailbox here by that name (#5.1.1 - chkusr)
      next if e[0, 1].ord <  48 || e[0, 1].ord > 122
      next if v['diagnosis'].empty? == false
      v['diagnosis'] = e
    end
  end
  return nil if recipients == 0
  return {"ds" => dscontents, "rfc822" => emailparts[1]}
end