Module: Sisimai::Lhost::Biglobe

Defined in:
lib/sisimai/lhost/biglobe.rb

Overview

Sisimai::Lhost::Biglobe decodes a bounce email which created by BIGLOBE www.biglobe.ne.jp/. Methods in the module are called from only Sisimai::Message.

Constant Summary collapse

Indicators =
Sisimai::Lhost.INDICATORS
Boundaries =
['Content-Type: message/rfc822'].freeze
StartingOf =
{
  message: ['   ----- The following addresses had delivery problems -----'],
  error:   ['   ----- Non-delivered information -----'],
}.freeze

Class Method Summary collapse

Class Method Details

.descriptionObject



77
# File 'lib/sisimai/lhost/biglobe.rb', line 77

def description; return 'BIGLOBE: https://www.biglobe.ne.jp'; end

.inquire(mhead, mbody) ⇒ Hash, Nil

Parameters:

  • mhead (Hash)

    Message headers of a bounce email

  • mbody (String)

    Message body of a bounce email

Returns:

  • (Hash)

    Bounce data list and message/rfc822 part

  • (Nil)

    it failed to decode or the arguments are missing



20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
# File 'lib/sisimai/lhost/biglobe.rb', line 20

def inquire(mhead, mbody)
  return nil if mhead['from'].include?('postmaster@') == false
  return nil if %w[biglobe inacatv tmtv ttv].none? { |a| mhead['from'].include?('@' + a + '.ne.jp') }
  return nil if mhead['subject'].start_with?('Returned mail:') == false

  dscontents = [Sisimai::Lhost.DELIVERYSTATUS]; v = nil
  emailparts = Sisimai::RFC5322.part(mbody, Boundaries)
  bodyslices = emailparts[0].split("\n")
  readcursor = 0      # (Integer) Points the current cursor position
  recipients = 0      # (Integer) The number of 'Final-Recipient' header

  while e = bodyslices.shift do
    # Read error messages and delivery status lines from the head of the email to the previous
    # line of the beginning of the original message.
    if readcursor == 0
      # Beginning of the bounce message or delivery status part
      readcursor |= Indicators[:deliverystatus] if e == StartingOf[:message][0]
      next
    end
    next if (readcursor & Indicators[:deliverystatus]) == 0 || e.empty?

    # This is a MIME-encapsulated message.
    #
    # ----_Biglobe000000/00000.biglobe.ne.jp
    # Content-Type: text/plain; charset="iso-2022-jp"
    #
    #    ----- The following addresses had delivery problems -----
    # ********@***.biglobe.ne.jp
    #
    #    ----- Non-delivered information -----
    # The number of messages in recipient's mailbox exceeded the local limit.
    #
    # ----_Biglobe000000/00000.biglobe.ne.jp
    # Content-Type: message/rfc822
    #
    v = dscontents[-1]

    if e.include?('@') && e.include?(' ') == false
      #    ----- The following addresses had delivery problems -----
      # ********@***.biglobe.ne.jp
      if v["recipient"] != ""
        # There are multiple recipient addresses in the message body.
        dscontents << Sisimai::Lhost.DELIVERYSTATUS
        v = dscontents[-1]
      end

      next if Sisimai::Address.is_emailaddress(e) == false
      v['recipient'] = e
      recipients += 1
    else
      next if e.include?('--')
      v['diagnosis'] += "#{e }"
    end
  end
  return nil if recipients == 0
  return { 'ds' => dscontents, 'rfc822' => emailparts[1] }
end