Module: Sisimai::RFC5965

Defined in:
lib/sisimai/rfc5965.rb

Overview

Sisimai::RFC5965 - A class for An Extensible Format for Email Feedback Reports

Class Method Summary collapse

Class Method Details

.FIELDINDEXObject



6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
# File 'lib/sisimai/rfc5965.rb', line 6

def FIELDINDEX
  return [
    # Required Fields
    # The following report header fields MUST appear exactly once:
    'Feedback-Type', 'User-Agent', 'Version',

    # Optional Fields Appearing Once
    # The following header fields are optional and MUST NOT appear more than once:
    # - "Reporting-MTA" is defined in Sisimai::RFC1894->FIELDINDEX()
    'Original-Envelope-Id', 'Original-Mail-From', 'Arrival-Date', 'Source-IP', 'Incidents',

    # Optional Fields Appearing Multiple Times
    # The following set of header fields are optional and may appear any number of times as
    # appropriate:
    'Authentication-Results', 'Original-Rcpt-To', 'Reported-Domain', 'Reported-URI',

    # The historic field "Received-Date" SHOULD also be accepted and interpreted identically
    # to "Arrival-Date".  However, if both are present, the report is malformed and SHOULD be
    # treated as described in Section 4.
    'Received-Date',
  ]
end