Module: Sisimai::Reason::OnHold

Defined in:
lib/sisimai/reason/onhold.rb

Overview

Sisimai::Reason::OnHold checks the bounce reason is "OnHold" or not. This class is called only Sisimai::Reason class. Sisimai will set "OnHold" to the reason of email bounce if there is no (or less) detailed information about email bounce for judging the reason.

Class Method Summary collapse

Class Method Details

.descriptionObject



10
# File 'lib/sisimai/reason/onhold.rb', line 10

def description; return 'Sisimai could not decided the reason due to there is no (or less) detailed information for judging the reason'; end

.match(_argv1) ⇒ Boolean

Try to match that the given text and regular expressions

Parameters:

  • argv1 (String)

    String to be matched with regular expressions

Returns:

  • (Boolean)

    false: Did not match, true: Matched



15
# File 'lib/sisimai/reason/onhold.rb', line 15

def match(_argv1); return false; end

.textObject



9
# File 'lib/sisimai/reason/onhold.rb', line 9

def text; return Sisimai::Eb::Re___1; end

.true(argvs) ⇒ Boolean

On hold, Could not decide the bounce reason...

Parameters:

Returns:

  • (Boolean)

    true: Status code is "OnHold" false: is not "OnHold"

See Also:



22
23
24
25
26
27
# File 'lib/sisimai/reason/onhold.rb', line 22

def true(argvs)
  return false if argvs['deliverystatus'].empty?
  return true  if argvs['reason'] == Sisimai::Eb::Re___1
  return true  if Sisimai::SMTP::Status.name(argvs['deliverystatus']) == Sisimai::Eb::Re___1
  return false
end