Module: Sisimai::Reason::Suppressed

Defined in:
lib/sisimai/reason/suppressed.rb

Overview

Sisimai::Reason::Suppressed checks the bounce reason is "Suppressed" or not. This class is called only Sisimai::Reason class.

This is the error that the recipient adddress is listed in the suppression list of the relay server, and was not delivered.

Class Method Summary collapse

Class Method Details

.descriptionObject



12
# File 'lib/sisimai/reason/suppressed.rb', line 12

def description; return "Email was not delivered due to being listed in the suppression list of MTA"; end

.match(argv1) ⇒ Boolean

Try to match that the given text and regular expressions

Parameters:

  • argv1 (String)

    String to be matched with regular expressions

Returns:

  • (Boolean)

    false: Did not match, true: Matched



17
# File 'lib/sisimai/reason/suppressed.rb', line 17

def match(argv1); return false; end

.textObject



11
# File 'lib/sisimai/reason/suppressed.rb', line 11

def text; return Sisimai::Eb::ReSTOP; end

.true(argvs) ⇒ Boolean

Whether the address is listed in the suppression list

Parameters:

Returns:

  • (Boolean)

    true: The address is listed in the suppression list false: is not listed in the suppression list

See Also:



24
25
26
27
# File 'lib/sisimai/reason/suppressed.rb', line 24

def true(argvs)
  return true if argvs['reason'] == Sisimai::Eb::ReSTOP
  return match(argvs['diagnosticcode'].downcase)
end