Module: Sisimai::Reason::SyntaxError

Defined in:
lib/sisimai/reason/syntaxerror.rb

Overview

Sisimai::Reason::SyntaxError checks the bounce reason is "SyntaxError" or not. This class is called only Sisimai::Reason class.

This is the error that a destination mail server could not recognize SMTP command which is sent from a sender's MTA. Sisimai will set "SyntaxError" to the reason if the value of "replycode" begins with "50" such as 502, or 503. Action: failed Status: 5.5.0 Diagnostic-Code: SMTP; 503 Improper sequence of commands

Class Method Summary collapse

Class Method Details

.descriptionObject



17
# File 'lib/sisimai/reason/syntaxerror.rb', line 17

def description; return 'Email rejected due to syntax error at sent commands in SMTP session'; end

.matchObject



18
# File 'lib/sisimai/reason/syntaxerror.rb', line 18

def match(*); return false; end

.textObject



16
# File 'lib/sisimai/reason/syntaxerror.rb', line 16

def text; return Sisimai::Eb::ReCOMM; end

.true(argvs) ⇒ Boolean

Connection rejected due to syntax error or not

Parameters:

Returns:

  • (Boolean)

    true: Connection rejected due to syntax error false: is not syntax error

See Also:

Since:

  • 4.1.25



26
# File 'lib/sisimai/reason/syntaxerror.rb', line 26

def true(argvs); return false; end