Module: Sisimai::Rhost::KDDI

Defined in:
lib/sisimai/rhost/kddi.rb

Overview

Sisimai::Rhost detects the bounce reason from the content of Sisimai::Fact object as an argument of find() method when the value of “rhost” of the object is “lsean.ezweb.ne.jp” or “msmx.au.com”. This class is called only Sisimai::Fact class.

Constant Summary collapse

MessagesOf =
{
  'filtered'    => '550 : user unknown',  # The response was: 550 : User unknown
  'userunknown' => '>: user unknown',     # The response was: 550 <...>: User unknown
}.freeze

Class Method Summary collapse

Class Method Details

.find(argvs) ⇒ String

Detect bounce reason from au (KDDI)

Parameters:

Returns:

  • (String)

    The bounce reason for au.com or ezweb.ne.jp

Since:

  • v4.22.6



17
18
19
20
21
22
23
24
25
26
27
28
29
30
# File 'lib/sisimai/rhost/kddi.rb', line 17

def find(argvs)
  return "" if argvs["diagnosticcode"].empty?
  issuedcode = argvs['diagnosticcode'].downcase
  reasontext = ''

  MessagesOf.each_key do |e|
    # Try to match the error message with message patterns defined in $MessagesOf
    next if issuedcode.include?(MessagesOf[e]) == false
    reasontext = e
    break
  end

  return reasontext
end