Module: Sisimai::Lhost::X3
- Defined in:
- lib/sisimai/lhost/x3.rb
Overview
Sisimai::Lhost::X3 decodes a bounce email which created by Unknown MTA #3. Methods in the module are called from only Sisimai::Message.
Constant Summary collapse
- Indicators =
Sisimai::Lhost.INDICATORS
- Boundaries =
['Content-Type: message/rfc822'].freeze
- StartingOf =
{message: [' This is an automatically generated Delivery Status Notification.']}.freeze
Class Method Summary collapse
Class Method Details
.description ⇒ Object
86 |
# File 'lib/sisimai/lhost/x3.rb', line 86 def description; return 'Unknown MTA #3'; end |
.inquire(mhead, mbody) ⇒ Hash, Nil
This method is abstract.
Decodes the bounce message from Unknown MTA #3
17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 |
# File 'lib/sisimai/lhost/x3.rb', line 17 def inquire(mhead, mbody) return nil if mhead['subject'].start_with?('Delivery status notification') == false return nil if mhead['from'].start_with?('Mail Delivery System') == false require 'sisimai/smtp/command' dscontents = [Sisimai::Lhost.DELIVERYSTATUS] emailparts = Sisimai::RFC5322.part(mbody, Boundaries) bodyslices = emailparts[0].split("\n") readcursor = 0 # (Integer) Points the current cursor position recipients = 0 # (Integer) The number of 'Final-Recipient' header v = nil while e = bodyslices.shift do # Read error messages and delivery status lines from the head of the email to the previous # line of the beginning of the original message. if readcursor == 0 # Beginning of the bounce message or delivery status part readcursor |= Indicators[:deliverystatus] if e.start_with?(StartingOf[:message][0]) next end next if (readcursor & Indicators[:deliverystatus]) == 0 || e.empty? # ============================================================================ # This is an automatically generated Delivery Status Notification. # # Delivery to the following recipients failed permanently: # # * kijitora@example.com # # # ============================================================================ # Technical details: # # SMTP:RCPT host 192.0.2.8: 553 5.3.0 <kijitora@example.com>... No such user here # # # ============================================================================ v = dscontents[-1] if e.include?(' * ') && e.include?('@') # * kijitora@example.com if v["recipient"] != "" # There are multiple recipient addresses in the message body. dscontents << Sisimai::Lhost.DELIVERYSTATUS v = dscontents[-1] end v['recipient'] = e[e.index(' * ') + 3, e.size] recipients += 1 else # Detect error message case when e.start_with?('SMTP:') # SMTP:RCPT host 192.0.2.8: 553 5.3.0 <kijitora@example.com>... No such user here v['command'] = Sisimai::SMTP::Command.find(e) v['diagnosis'] = e when e.start_with?('Routing: ') # Routing: Could not find a gateway for kijitora@example.co.jp v['diagnosis'] = e[9, e.size] when e.start_with?('Diagnostic-Code: smtp; ') # Diagnostic-Code: smtp; 552 5.2.2 Over quota v['diagnosis'] = e[e.index(';') + 2, e.size] end end end return nil if recipients == 0 return {"ds" => dscontents, "rfc822" => emailparts[1]} end |