Module: Sisimai::Reason::Delivered
- Defined in:
- lib/sisimai/reason/delivered.rb
Overview
Sisimai::Reason::Delivered checks the email you sent is delivered successfully or not by matching diagnostic messages with message patterns. Sisimai will set "Delivered" to the value of "reason" when Status: field in the bounce message begins with "2" like following:
Final-Recipient: rfc822; kijitora@neko.nyaan.jp Action: delivered Status: 2.1.5 Diagnostic-Code: SMTP; 250 2.1.5 OK
This class is called only Sisimai.reason method. This is NOT AN ERROR reason.
Class Method Summary collapse
Class Method Details
.description ⇒ Object
17 |
# File 'lib/sisimai/reason/delivered.rb', line 17 def description; return 'Email delivered successfully'; end |
.match ⇒ Object
18 |
# File 'lib/sisimai/reason/delivered.rb', line 18 def match; return false; end |
.text ⇒ Object
16 |
# File 'lib/sisimai/reason/delivered.rb', line 16 def text; return Sisimai::Eb::ReSENT; end |
.true ⇒ Object
19 |
# File 'lib/sisimai/reason/delivered.rb', line 19 def true(*); return false; end |