Module: Sisimai::Reason::Suppressed
- Defined in:
- lib/sisimai/reason/suppressed.rb
Overview
Sisimai::Reason::Suppressed checks the bounce reason is "Suppressed" or not. This class is called only Sisimai::Reason class.
This is the error that the recipient adddress is listed in the suppression list of the relay server, and was not delivered.
Class Method Summary collapse
- .description ⇒ Object
-
.match(argv1) ⇒ Boolean
Try to match that the given text and regular expressions.
- .text ⇒ Object
-
.true(argvs) ⇒ Boolean
Whether the address is listed in the suppression list.
Class Method Details
.description ⇒ Object
12 |
# File 'lib/sisimai/reason/suppressed.rb', line 12 def description; return "Email was not delivered due to being listed in the suppression list of MTA"; end |
.match(argv1) ⇒ Boolean
Try to match that the given text and regular expressions
17 |
# File 'lib/sisimai/reason/suppressed.rb', line 17 def match(argv1); return false; end |