Module: Sisimai::Reason::SyntaxError
- Defined in:
- lib/sisimai/reason/syntaxerror.rb
Overview
Sisimai::Reason::SyntaxError checks the bounce reason is “syntaxerror” or not. This class is called only Sisimai::Reason class.
This is the error that a destination mail server could not recognize SMTP command which is sent from a sender’s MTA. Sisimai will set “syntaxerror” to the reason if the value of “replycode” begins with “50” such as 502, or 503.
Action: failed
Status: 5.5.0
Diagnostic-Code: SMTP; 503 Improper sequence of commands
Class Method Summary collapse
- .description ⇒ Object
- .match ⇒ Object
- .text ⇒ Object
-
.true(argvs) ⇒ Boolean
Connection rejected due to syntax error or not.
Class Method Details
.description ⇒ Object
16 |
# File 'lib/sisimai/reason/syntaxerror.rb', line 16 def description; return 'Email rejected due to syntax error at sent commands in SMTP session'; end |
.match ⇒ Object
17 |
# File 'lib/sisimai/reason/syntaxerror.rb', line 17 def match(*); return false; end |
.text ⇒ Object
15 |
# File 'lib/sisimai/reason/syntaxerror.rb', line 15 def text; return 'syntaxerror'; end |
.true(argvs) ⇒ Boolean
Connection rejected due to syntax error or not
25 |
# File 'lib/sisimai/reason/syntaxerror.rb', line 25 def true(argvs); return false; end |