Module: Sisimai::Reason::SystemFull
- Defined in:
- lib/sisimai/reason/systemfull.rb
Overview
Sisimai::Reason::SystemFull checks the bounce reason is “systemfull” or not. This class is called only Sisimai::Reason class.
This is the error that a destination mail server’s disk (or spool) is full. Sisimai will set systemfull to the reason of email bounce if the value of Status: field in a bounce email is “4.3.1” or “5.3.1”.
Constant Summary collapse
- Index =
[ 'exceeded storage allocation', # MS Exchange 'mail system full', ].freeze
Class Method Summary collapse
- .description ⇒ Object
-
.match(argv1) ⇒ Boolean
Try to match that the given text and regular expressions.
- .text ⇒ Object
-
.true(_argvs) ⇒ Boolean
The bounce reason is system full or not.
Class Method Details
.description ⇒ Object
17 |
# File 'lib/sisimai/reason/systemfull.rb', line 17 def description; return "Email rejected due to a destination mail server's disk is full"; end |
.match(argv1) ⇒ Boolean
Try to match that the given text and regular expressions
22 23 24 25 26 |
# File 'lib/sisimai/reason/systemfull.rb', line 22 def match(argv1) return false if argv1.nil? || argv1.empty? return true if Index.any? { |a| argv1.include?(a) } return false end |
.text ⇒ Object
16 |
# File 'lib/sisimai/reason/systemfull.rb', line 16 def text; return 'systemfull'; end |
.true(_argvs) ⇒ Boolean
The bounce reason is system full or not
33 |
# File 'lib/sisimai/reason/systemfull.rb', line 33 def true(_argvs); return false; end |